/* Copyright.h */

/*  $RCSfile: copyright.h,v $
 *  $Revision: 1.4 $
 *  $Date: 1994/06/01 22:20:09 $
 */

/*
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Copyright (c) 1992, 1993 Mike Gleason, NCEMRSoft.
 * Copyright (c) 1985, 1989 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted provided
 * that: (1) source distributions retain this entire copyright notice and
 * comment, and (2) distributions may not be sold for profit on physicaical
 * media such as disks, tapes, and CD-ROMS, without expressed written
 * permission.
 */

#ifdef _main_c_
#ifndef lint
static char copyright[] = "@(#) Copyright (c) 1992, 1993 by NCEMRSoft and Copyright (c) 1985, 1989 Regents of the University of California.\n All rights reserved.\n";
#endif /* not lint */
#endif /* _main_c_ */

/* eof copyright.h */
